package sharecode.listeners;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.ecf.datashare.IChannel;
import org.eclipse.ecf.datashare.IChannelListener;
import org.eclipse.ecf.datashare.events.IChannelEvent;
import org.eclipse.ecf.datashare.events.IChannelMessageEvent;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import sharecode.asynchronous.MessageProducer;
import sharecode.asynchronous.QueueSession;
import sharecode.asynchronous.RequestQueue;
import sharecode.core.Activator;
import sharecode.core.model.FileMessage;
import sharecode.core.model.Input;
import sharecode.core.model.Result;
import sharecode.exception.ShareCodeException;
import sharecode.message.CancelMessage;
import sharecode.message.ReplayMessage;
import sharecode.message.RequestMessage;
import sharecode.message.SharedCodeMessage;
import sharecode.message.content.Content;
import sharecode.util.GeneratorUUID;
import sharecode.util.Tracer;
import sharecode.views.ShareCodeResultView;


public class PresenceChannelListener implements IChannelListener {

	private IChannel channel;

	public PresenceChannelListener() {
	}

	/**
	 * 
	 */
	public void handleChannelEvent(IChannelEvent event) {

		if (channel == null) {
			channel = Activator.getDefault().getPresenceChannel();
		}

		if (event instanceof IChannelMessageEvent) {

			IChannelMessageEvent channelMessage = (IChannelMessageEvent) event;

			ByteArrayInputStream bins = new ByteArrayInputStream(channelMessage
					.getData());
			ObjectInputStream ois;

			try {
				ois = new ObjectInputStream(bins);
				SharedCodeMessage message = (SharedCodeMessage) ois.readObject();

				processMessage(channelMessage, message);

			} catch (IOException e) {
				// TODO Tratar LOG
				Activator.getDefault().getLog().log(
						new Status(IStatus.ERROR, Activator.PLUGIN_ID, 0, e
								.getLocalizedMessage(), e));
			} catch (ClassNotFoundException e) {
				// TODO Tratar LOG
				Activator.getDefault().getLog().log(
						new Status(IStatus.ERROR, Activator.PLUGIN_ID, 0, e
								.getLocalizedMessage(), e));
			} catch (ShareCodeException e) {
				// TODO Tratar LOG
				Activator.getDefault().getLog().log(
						new Status(IStatus.ERROR, Activator.PLUGIN_ID, 0, e
								.getLocalizedMessage(), e));
			} finally {
				// TODO nao sei se realmente precisamos desse metodo
				Activator.getDefault().setListenerActive(true);

			}
		}
	}

	/**
	 * 
	 * @param channelMessage
	 * @param message
	 * @throws ShareCodeException
	 */
	private void processMessage(IChannelMessageEvent channelMessage,SharedCodeMessage message) throws ShareCodeException {

		
		// TODO analisar se isso pode ser melhorado
		/*
		 * starts search code
		 */
		if (message instanceof RequestMessage) {
			request(channelMessage, message);
		} else if (message instanceof ReplayMessage) {
			replay(channelMessage, message);
		} else if (message instanceof CancelMessage) {
			cancel(channelMessage, message);
		}
	}

	/**
	 * 
	 * @param channelMessage
	 * @param message
	 */
	private void cancel(IChannelMessageEvent channelMessage,SharedCodeMessage message) {
		// TODO Caso chegue um pedido de cancelamento de pesquisa de um
		// determinado request, o plugin ira retirar o pedido da fila, caso ele
		// ainda nao tenha sido
		// tratado

	}

	/**
	 * 
	 * @param channel
	 * @param message
	 * @throws ShareCodeException
	 */
	private void request(IChannelMessageEvent channel, SharedCodeMessage message)throws ShareCodeException {
		
		/*
		 * Adiciona informacoes a mensagem(pedido) que chegou Coloca o pedido de
		 * pesquisa em uma fila
		 */
		// Adiciona o ID do requestor
		message.setRequestorId(channel.getFromContainerID());

		// get the current session queue
		QueueSession session = QueueSession.instance();
		RequestQueue requestQueue = session.getRequestQueue();

		MessageProducer requestSender = session.createProducer(requestQueue);
		requestSender.send(message);
		
		Tracer.trace(this, "Receive a request from Requestor "+message.getRequestorId().getName()+" :: message "+message.getMessageId());
	}

	/**
	 * 
	 * @param channel
	 * @param message
	 * @throws ShareCodeException
	 */
	private void replay(IChannelMessageEvent channel, SharedCodeMessage message)throws ShareCodeException {
		Tracer.trace(this, "Receive message response "+message.getMessageId()+" from Replier "+message.getRequestorId().getName()+" referent the message "+message.getCorrelationId());
		
		Content content = message.getContent();
		List resultados = content.getResults();
		
		ShareCodeResultView sv = ShareCodeResultView.getInstance(message.getCorrelationId());
		
		sv.show(resultados);
		
		

	}
}
