package sharecode.util;

import java.net.InetAddress;

import java.net.UnknownHostException;

import java.security.SecureRandom;

import sharecode.exception.ShareCodeException;





/**

 *

 * Gera uma chave nica para ser utilizada 

 * na criao de novos registros em tabelas.

 * 

 * 

 * @date   17/12/2002

 **/

public class GeneratorUUID {

	// Instncia do singleton

	private static GeneratorUUID instance;

	

	//secure random para prover semente no repetitiva

	private SecureRandom seeder;

	

	//valor central da chave em cache (ip + hash do objeto)

	private String midValue;



	/**

	 * Construtor privado

 	 */

	private GeneratorUUID() throws ShareCodeException{

		try {

			//Recupera o identificador do n

			InetAddress inet = InetAddress.getLocalHost();

			byte[] bytes = inet.getAddress();

			

			String hexInetAddress = hexFormat(getInt(bytes),8);

			

			//Recupera o hashcode deste objeto

			String thisHashCode = hexFormat(System.identityHashCode(this),8);

			

			//Grava o valor central da chaved

			this.midValue = hexInetAddress + thisHashCode;

			

			//Carrega o randomizer

			seeder = new SecureRandom();

			seeder.nextInt();

		} catch (UnknownHostException e) {

			throw new ShareCodeException(e);

		}

		

	}

	

	/**

	 * Retorna a instncia nica de GeradorPK

	 */

	public static GeneratorUUID getInstance() throws ShareCodeException {

		String s = "sync";

		synchronized(s) {

			if (instance == null) {

				instance = new GeneratorUUID();

			}

		}

		

		return instance;

	}

		

	/**

	 * Retorna a representao em inteiro de um byte array

	 */

    private int getInt(byte abyte[]) {

        int i = 0;

        int j = 24;

        for(int k = 0; j >= 0; k++){

            int l = abyte[k] & 0xff;

            i += l << j;

            j -= 8;

        }



        return i;

    }



	/**

	 * Retorna a representao hexadecimal de um inteiro i, limitada ao

	 * tamanho especificado.

	 */

    private String hexFormat(int i, int tamanho) {

        String s = Integer.toHexString(i);

        return padHex(s, tamanho) + s;

    }



	/**

	 * Completa um string representando um nmero com zeros  esquerda, at

	 * atingir o tamanho especificado

	 */

    private String padHex(String numero, int tamanho){

        StringBuffer stringbuffer = new StringBuffer();

        if(numero.length() < tamanho)

        {

            for(int j = 0; j < tamanho - numero.length(); j++)

                stringbuffer.append("0");



        }

        return stringbuffer.toString();

    }



	/**

	 * Retorna uma string nica a ser utilizada como identificador.

	 */

	public String geraUUID(){

		long timeNow = System.currentTimeMillis();

		
		int timeLow = (int) timeNow & 0xFFFFFFFF;


		int node = seeder.nextInt();

		

		return (hexFormat(timeLow,8) + midValue + hexFormat(node,8));

	}

	


}


