package sharecode.util;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.internal.core.JavaProject;
import org.eclipse.search.internal.ui.text.FileSearchQuery;
import org.eclipse.search.ui.text.AbstractTextSearchResult;
import org.eclipse.search.ui.text.FileTextSearchScope;

import sharecode.core.Activator;
import sharecode.core.model.FileMessage;
import sharecode.core.model.Input;
import sharecode.core.model.preferences.PreferenceConstants;
import sharecode.core.model.preferences.Preferences;

public class SearchUtil {
	
	public static List find(Input input, IProgressMonitor progressMonitor){
		
		
		IProject[] projects = ProjectListElement.filterProjects(input.getLicense());
 		
		List selectedProjects = new ArrayList();
		
		
		for(int i=0; i<projects.length; i++) {
			if (JavaProject.hasJavaNature(projects[i])) {
				IResource javaProject = (IResource) projects[i].getAdapter(IResource.class);
				selectedProjects.add(javaProject);	
			}
		}
		IResource[] projetosJava = new IResource[selectedProjects.size()];
		selectedProjects.toArray(projetosJava);

		FileTextSearchScope fss = FileTextSearchScope.newSearchScope(projetosJava, new String[]{"*.java"}, false);
		
		String pattern = "";
		
		if (input.isFlagImport()) {
			   pattern="(^\\s*import\\s*.*#_V_A_L_U_E_#)";
		}
		if (input.isFlagClass()) {
		   pattern+= (pattern.length()>0?"|":"") + "(((.*\\})|(.*\\*/)|(^\\s*))\\s*(public|private|protected|(\\s*))\\s*class\\s*#_V_A_L_U_E_#)";
		}
		if (input.isFlagComments()) {
		   pattern+= (pattern.length()>0?"|":"") + "((?:/\\*(?:[^*]|(?:\\*+[^*/]))*#_V_A_L_U_E_#(?:[^*]|(?:\\*+[^*/]))*\\*+/)|(//.*#_V_A_L_U_E_#.*$))";
		}
		if (input.isFlagCodeBlock()) {
		   pattern+= (pattern.length()>0?"|":"") + "(\\{([^{]+#_V_A_L_U_E_#[^}]+|\\{|\\})*\\})";
		}
		String expression = makeExpression(pattern, input.getExpression());
		
		FileSearchQuery fsq = new FileSearchQuery(expression, true, false, fss);
		fsq.run(progressMonitor);
		AbstractTextSearchResult resultado = (AbstractTextSearchResult)fsq.getSearchResult();
		List retorno =  new ArrayList();
		Object[] obs = resultado.getElements();
		for (int i = 0; i < obs.length; i++) {
			IFile arq = (IFile) obs[i];

			Date dt = new Date(arq.getLocalTimeStamp());
			if (input.getDate()!=null && input.getDate().before(dt))
				continue;
			
			FileMessage scfile = createSCFile(arq);
			scfile.setExpression(input.getExpression());
			scfile.setLicense(input.getLicense());
			retorno.add(scfile);
		}
		return retorno;

	}
	private static FileMessage createSCFile( IFile arq) {
		FileMessage scfile = new FileMessage();
		scfile.setFileName(arq.getName());
		
		
		
		scfile.setDate(new Date(arq.getLocalTimeStamp()));
		try {
			ByteArrayOutputStream bo = new ByteArrayOutputStream();
			BufferedInputStream is = new BufferedInputStream(arq.getContents());
			int c = -1;
			while ( (c = is.read())>-1)
				bo.write(c);
			is.close();
			scfile.setData(bo.toByteArray());
			bo.close();
			String tmp = new String(scfile.getData());
			Pattern p1 = Pattern.compile("(((.*\\})|(.*\\*/)|(^))\\s+(public|private|protected|\\s+))\\s*class\\s*[^ ]*(\\s|\\{)" , Pattern.MULTILINE);
			Matcher m = p1.matcher(tmp);
			if (m.find()) {
				String nmClass = m.group();
				int pos = nmClass.indexOf("class")+5 ;
				nmClass = nmClass.substring(pos).trim();
				pos = nmClass.indexOf(" ");
				if (pos==-1) pos = nmClass.indexOf("{");
				if (pos>-1)
					nmClass= nmClass.substring(0,pos);
				pos = tmp.indexOf("package");
				if (pos>-1){
					int posf = tmp.indexOf(";", pos + 7);
					//posf = posf - (pos + 7);
					posf = posf - pos;
					//TODO ver o problema que esta ocorrendo para obter o packageName
					/*
					 * java.lang.StringIndexOutOfBoundsException: String index out of range: -26
						at java.lang.String.substring(String.java:1768)
						at sharecode.util.SearchUtil.createSCFile(SearchUtil.java:118)
						at sharecode.util.SearchUtil.find(SearchUtil.java:80)
						at sharecode.asynchronous.MessageConsumer.consume(MessageConsumer.java:47)
						at sharecode.asynchronous.job.TimerConsumerJob.run(TimerConsumerJob.java:36)
						at org.eclipse.core.internal.jobs.Worker.run(Worker.java:58)
					 */
					String packageName = "";//tmp.substring(pos+7, posf).trim();
					nmClass = packageName +  "" + nmClass;
							}
				scfile.setClassName(nmClass);
			}
			

			
			String from = Activator.getDefault().getPreferences().getLocalName();
			scfile.setFrom(from);
			
		} catch (CoreException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return scfile;
	}
	private static String makeExpression(String pattern, String text) {
		String formato = pattern;
		int pos=-1;
		do {
			pos = formato.indexOf("#_V_A_L_U_E_#", pos+1) ;
			if (pos>-1)
				formato = formato.substring(0,pos) + text + formato.substring(pos + 13);
			
		} while (pos>-1);
		return formato;
	}

	

}
