package sharecode.views;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import sharecode.actions.DisplayFileAction;
import sharecode.core.SCMessages;
import sharecode.core.model.FileMessage;
import sharecode.util.Util;

/**
 * This sample class demonstrates how to plug-in a new workbench view. The view
 * shows data obtained from the model. The sample creates a dummy model on the
 * fly, but a real implementation would connect to the model available either in
 * this or another plug-in (e.g. the workspace). The view is connected to the
 * model using a content provider.
 * <p>
 * The view uses a label provider to define how model objects should be
 * presented in the view. Each view can present the same model objects using
 * different labels and icons, if needed. Alternatively, a single label provider
 * can be shared between views in order to ensure that objects of the same type
 * are presented in the same way everywhere.
 * <p>
 */

public class ShareCodeResultView extends ViewPart implements Runnable{
	private List results;
	
	private TableViewer viewer;

	private List resultados;
	
	public void run() {
		    try {
				getSite().getWorkbenchWindow().getActivePage().showView("sharecode.views.ResultView");
			} catch (PartInitException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			this.addResults(resultados);
			this.viewer.refresh();
			

	}
	
	public void show(List resultados) {	
		this.resultados = resultados;
		getSite().getWorkbenchWindow().getWorkbench().getDisplay().syncExec(this);
	}

	/*
	 * The content provider class is responsible for providing objects to the
	 * view. It can wrap existing objects in adapters or simply return objects
	 * as-is. These objects may be sensitive to the current input of the view,
	 * or ignore it and always show the same content (like Task List, for
	 * example).
	 */

	class ViewContentProvider implements IStructuredContentProvider {
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}

		public void dispose() {
		}

		public Object[] getElements(Object parent) {
			if (results !=null)
			   return results.toArray();
			return null;
		}
	}

	class ViewLabelProvider extends LabelProvider implements
			ITableLabelProvider {

		private SimpleDateFormat sd;

		public ViewLabelProvider() {
			sd = new SimpleDateFormat();
		}

		public String getColumnText(Object obj, int index) {
			FileMessage vl = (FileMessage) obj ;
			Object o=null;
			switch (index) {
			case 1:
				o=vl.getFileName();
				break;
			case 2:
				o=vl.getClassName();
				break;
			case 3:
				o = sd.format(vl.getDate());
				break;
			case 4:
				o=vl.getFrom();
				break;
			case 5:
				o=vl.getExpression();
				break;
			case 6:
				o=vl.getLicense();
				break;

			default:
				break;
			}
			return getText(o);
		}

		public Image getColumnImage(Object obj, int index) {
		    if (index != 0){
		      return null;
		    }else{
		    	//TODO fazer o tratamento para pegar o icone file ou fileJava(ver a classe Util)
		      return Util.getImageRegistry().get("file");
		    }

		}

		public Image getImage(Object obj) {
			return PlatformUI.getWorkbench().getSharedImages().getImage(
					ISharedImages.IMG_OBJ_ELEMENT);
		}
	}

	class NameSorter extends ViewerSorter {
		public int compare(Viewer viewer, Object e1, Object e2){
			return ((FileMessage)e1).getExpression().compareTo(((FileMessage)e2).getExpression());
		}
	}

	/**
	 * The constructor.
	 */
	public ShareCodeResultView() {
		results = new ArrayList();
	}
	
	private static Map views = new HashMap();
	
	//TODO precisa ver se realmente precisamos armazenar a view em map. Pois acho necessario somente uma unica view
	public static ShareCodeResultView getInstance(String id) {
		ShareCodeResultView view = null;
		if (views.containsKey(id)){
			view = (ShareCodeResultView )views.get(id);
		} else {
			IWorkbenchWindow ww = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
			IWorkbenchPage wp = ww.getActivePage();
			view = (ShareCodeResultView)wp.findView("sharecode.views.ResultView");
				//view = (ShareCodeResultView )Workbench.getInstance().getActiveWorkbenchWindow().getActivePage().findView("sharecode.views.ResultView");
			if (view ==null )
				try {
					view = (ShareCodeResultView)wp.showView("sharecode.views.ResultView");
					//view = (ShareCodeResultView )Workbench.getInstance().getActiveWorkbenchWindow().getActivePage().showView("sharecode.views.ResultView");
				} catch (PartInitException e) {
					// TODO Tratar log
					e.printStackTrace();
				}
			views.put(id, view);

		}
		return view;
	}
	

	
	public void addResults(List newResults){
		results.addAll(newResults);
		viewer.getTable().redraw();
	}

	/**
	 * This is a callback that will allow us to create the viewer and initialize
	 * it.
	 */
	public void createPartControl(Composite parent) {
		viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL);

		TableColumn c = new TableColumn(viewer.getTable(), SWT.LEFT, 0);
		c.setText(""); 
		c.setWidth(20);

		c = new TableColumn(viewer.getTable(), SWT.LEFT, 1);
		c.setText(SCMessages.getString("sc.view.column.file")); 
		c.setWidth(200);
		
		c = new TableColumn(viewer.getTable(), SWT.LEFT, 2);
		c.setText(SCMessages.getString("sc.view.column.class")); 
		c.setWidth(200);
		c = new TableColumn(viewer.getTable(), SWT.LEFT, 3);
		c.setText(SCMessages.getString("sc.view.column.date")); 
		c.setWidth(200);
		
		c = new TableColumn(viewer.getTable(), SWT.LEFT, 4);
		c.setText(SCMessages.getString("sc.view.column.from")); 
		c.setWidth(200);
		
		c = new TableColumn(viewer.getTable(), SWT.LEFT, 5);
		c.setText(SCMessages.getString("sc.view.column.expression")); 
		c.setWidth(200);

		c = new TableColumn(viewer.getTable(), SWT.LEFT, 6);
		c.setText(SCMessages.getString("sc.view.column.license")); 
		c.setWidth(200);
		
		viewer.setContentProvider(new ViewContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider());
		viewer.setSorter(new NameSorter());
		viewer.setInput(getViewSite());

		viewer.getTable().redraw();
		viewer.getTable().setHeaderVisible(true);
		viewer.getTable().setLinesVisible(true);

		//makeActions();
		hookDoubleClickAction();
	}


	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				ISelection selection = viewer.getSelection();				
				final FileMessage scpFile = (FileMessage) ((IStructuredSelection)selection).getFirstElement();
				DisplayFileAction fileAction = new DisplayFileAction(scpFile);
				fileAction.run();
			}
		});
	}



	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		viewer.getControl().setFocus();
	}
}